@@ -22,23 +22,15 @@
 
 #include "navigatoritem.h"
 #include "docentry.h"
+#include "khc_debug.h"
 
-#include <KConfig>
-#include <KConfigGroup>
-#include <KSharedConfig>
-#include <KIconLoader>
 #include <KLocalizedString>
 
 #include <QFile>
 #include <QTextStream>
-#include <QLoggingCategory>
-#include <QPixmap>
+#include <QIcon>
 
-#include <stdlib.h>  // for getenv()
-
-namespace {
-QLoggingCategory category("org.kde.khelpcenter");
-}
+#include <prefs.h>
 
 using namespace KHC;
 
@@ -47,7 +39,7 @@ class InfoCategoryItem : public NavigatorItem
   public:
     InfoCategoryItem( NavigatorItem *parent, const QString &text );
 	
-    virtual void setExpanded( bool open );
+    virtual void itemExpanded( bool open );
 };
 
 class InfoNodeItem : public NavigatorItem
@@ -61,23 +53,24 @@ InfoCategoryItem::InfoCategoryItem( NavigatorItem *parent, const QString &text )
 {
   setAutoDeleteDocEntry( true );
   setExpanded( false );
-//  qCDebug(category) << "Got category: " << text;
+  setIcon( 0, QIcon::fromTheme( "help-contents" ) );
+//  khcDebug() << "Got category: " << text;
 }
 
-void InfoCategoryItem::setExpanded( bool open )
+void InfoCategoryItem::itemExpanded( bool open )
 {
-  NavigatorItem::setExpanded( open );
+  NavigatorItem::itemExpanded( open );
 
-  if ( open && childCount() > 0 ) setIcon( 0, SmallIcon( "help-contents" ) );
+  if ( open && childCount() > 0 ) setIcon( 0, QIcon::fromTheme( "help-contents" ) );
 // TODO: was contents2 -> needs to be changed to help-contents-alternate or similar
-  else setIcon( 0, SmallIcon( "help-contents" ) );
+  else setIcon( 0, QIcon::fromTheme( "help-contents" ) );
 }
 
 InfoNodeItem::InfoNodeItem( InfoCategoryItem *parent, const QString &text )
   : NavigatorItem( new DocEntry( text ), parent )
 {
   setAutoDeleteDocEntry( true );
-//  qCDebug(category) << "Created info node item: " << text;
+//  khcDebug() << "Created info node item: " << text;
 }
 
 InfoTree::InfoTree( QObject *parent )
@@ -88,7 +81,7 @@ InfoTree::InfoTree( QObject *parent )
 
 void InfoTree::build( NavigatorItem *parent )
 {
-  qCDebug(category) << "Populating info tree.";
+  khcDebug() << "Populating info tree.";
 
   m_parentItem = parent;
 
@@ -99,21 +92,9 @@ void InfoTree::build( NavigatorItem *parent )
   m_categoryItem = new NavigatorItem( entry, parent );
   m_categoryItem->setAutoDeleteDocEntry( true );
 
-  KConfigGroup cfg(KSharedConfig::openConfig(), "Info pages");
-  QStringList infoDirFiles = cfg.readEntry( "Search paths" , QStringList() );
-  // Default paths taken fron kdebase/kioslave/info/kde-info2html.conf
-  if ( infoDirFiles.isEmpty() ) { 
-    infoDirFiles << "/usr/share/info";
-    infoDirFiles << "/usr/info";
-    infoDirFiles << "/usr/lib/info";
-    infoDirFiles << "/usr/local/info";
-    infoDirFiles << "/usr/local/lib/info";
-    infoDirFiles << "/usr/X11R6/info";
-    infoDirFiles << "/usr/X11R6/lib/info";
-    infoDirFiles << "/usr/X11R6/lib/xemacs/info";
-  }
+  QStringList infoDirFiles = Prefs::searchpaths();
 
-  QString infoPath = ::getenv( "INFOPATH" );
+  const QString infoPath = QFile::decodeName( qgetenv( "INFOPATH" ) );
   if ( !infoPath.isEmpty() )
     infoDirFiles += infoPath.split( ':');
 
@@ -130,7 +111,7 @@ void InfoTree::build( NavigatorItem *parent )
 
 void InfoTree::parseInfoDirFile( const QString &infoDirFileName )
 {
-  qCDebug(category) << "Parsing info dir file " << infoDirFileName;
+  khcDebug() << "Parsing info dir file " << infoDirFileName;
 
   QFile infoDirFile( infoDirFileName );
   if ( !infoDirFile.open( QIODevice::ReadOnly ) )
@@ -182,8 +163,15 @@ void InfoTree::parseInfoDirFile( const QString &infoDirFileName )
         item->entry()->setUrl( url );
       }
     }
+    catItem->sortChildren( 0, Qt::AscendingOrder );
   }
   infoDirFile.close();
+
+  foreach ( InfoCategoryItem *item, alphabSections ) {
+    item->sortChildren( 0, Qt::AscendingOrder );
+  }
+  m_alphabItem->sortChildren( 0, Qt::AscendingOrder );
+  m_categoryItem->sortChildren( 0, Qt::AscendingOrder );
 }
 
 
