@@ -22,28 +22,21 @@
 
 #include "searchwidget.h"
 
-#include <stdlib.h>
-#include <unistd.h>
-#include <errno.h>
-
 #include <QLabel>
-#include <QPushButton>
 #include <QComboBox>
-#include <QLayout>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
-#include <QBoxLayout>
-#include <QtDBus/QDBusConnection>
+#include <QHash>
 
 #include <KConfig>
-#include <KIconLoader>
 #include <KLocalizedString>
 
 #include "scopeitem.h"
 #include "docentrytraverser.h"
-#include "kcmhelpcenter.h"
-#include "prefs.h"
 #include "searchengine.h"
+#include "khc_debug.h"
+
+#include <prefs.h>
 
 namespace KHC {
 
@@ -51,8 +44,6 @@ SearchWidget::SearchWidget( SearchEngine *engine, QWidget *parent )
   : QWidget( parent ), mEngine( engine ),
   mScopeCount( 0 )
 {
-    QDBusConnection::sessionBus().registerObject("/SearchWidget", this, QDBusConnection::ExportScriptableSlots);
-
   QBoxLayout *topLayout = new QVBoxLayout( this );
   topLayout->setMargin( 2 );
   topLayout->setSpacing( 2 );
@@ -107,11 +98,6 @@ SearchWidget::SearchWidget( SearchEngine *engine, QWidget *parent )
   mScopeListView->setHeaderLabels( QStringList() << i18n("Scope") );
   topLayout->addWidget( mScopeListView, 1 );
 
-  QPushButton *indexButton = new QPushButton( i18n("Build Search &Index..."),
-                                              this );
-  connect( indexButton, SIGNAL( clicked() ), SIGNAL( showIndexDialog() ) );
-  topLayout->addWidget( indexButton );
-
 // FIXME: Use SearchHandler on double-clicked document
 #if 0
   connect( mScopeListView, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int) ),
@@ -142,7 +128,7 @@ void SearchWidget::readConfig( KConfig *cfg )
     KConfigGroup searchScopeGroup(cfg, "Custom Search Scope" );
     QTreeWidgetItemIterator it( mScopeListView );
     while( *it ) {
-      if ( (*it)->type() == ScopeItem::rttiId() ) {
+      if ( (*it)->type() == ScopeItem::ScopeItemType ) {
         ScopeItem *item = static_cast<ScopeItem *>(*it);
         item->setOn( searchScopeGroup.readEntry(
                          item->entry()->identifier(),
@@ -167,7 +153,7 @@ void SearchWidget::writeConfig( KConfig *cfg )
     KConfigGroup cg2 (cfg, "Custom Search Scope");
     QTreeWidgetItemIterator it( mScopeListView );
     while( (*it) ) {
-      if ( (*it)->type() == ScopeItem::rttiId() ) {
+      if ( (*it)->type() == ScopeItem::ScopeItemType ) {
         ScopeItem *item = static_cast<ScopeItem *>( (*it) );
         cg2.writeEntry( item->entry()->identifier(), item->isOn() );
       }
@@ -180,7 +166,7 @@ void SearchWidget::slotSwitchBoxes()
 {
   QTreeWidgetItemIterator it( mScopeListView );
   while( (*it) ) {
-    if ( (*it)->type() == ScopeItem::rttiId() ) {
+    if ( (*it)->type() == ScopeItem::ScopeItemType ) {
       ScopeItem *item = static_cast<ScopeItem *>( (*it) );
       item->setOn( !item->isOn() );
     }
@@ -194,7 +180,7 @@ void SearchWidget::scopeSelectionChanged( int id )
 {
   QTreeWidgetItemIterator it( mScopeListView );
   while( (*it) ) {
-    if ( (*it)->type() == ScopeItem::rttiId() ) {
+    if ( (*it)->type() == ScopeItem::ScopeItemType ) {
       ScopeItem *item = static_cast<ScopeItem *>( (*it) );
       bool state = item->isOn();
       switch( id ) {
@@ -220,7 +206,7 @@ void SearchWidget::scopeSelectionChanged( int id )
   checkScope();
 }
 
-QString SearchWidget::method()
+QString SearchWidget::method() const
 {
   QString m = "and";
   if ( mMethodCombo->currentIndex() == 1)
@@ -229,24 +215,23 @@ QString SearchWidget::method()
   return m;
 }
 
-int SearchWidget::pages()
+int SearchWidget::pages() const
 {
   int p = mPagesCombo->currentText().toInt();
 
   return p;
 }
 
-QString SearchWidget::scope()
+QStringList SearchWidget::scope() const
 {
-  QString scope;
+  QStringList scope;
 
   QTreeWidgetItemIterator it( mScopeListView );
   while( (*it) ) {
-    if ( (*it)->type() == ScopeItem::rttiId() ) {
+    if ( (*it)->type() == ScopeItem::ScopeItemType ) {
       ScopeItem *item = static_cast<ScopeItem *>( (*it) );
       if ( item->isOn() ) {
-        if ( !scope.isEmpty() ) scope += '&';
-        scope += QLatin1String("scope=") + item->entry()->identifier();
+        scope += item->entry()->identifier();
       }
     }
     ++it;
@@ -258,70 +243,61 @@ QString SearchWidget::scope()
 class ScopeTraverser : public DocEntryTraverser
 {
   public:
-    ScopeTraverser( SearchWidget *widget, int level ) :
-      mWidget( widget ), mLevel( level ), mParentItem( 0 ) {}
+    ScopeTraverser( SearchEngine *engine, QTreeWidgetItem *parentItem, int level ) :
+      mEngine( engine ), mLevel( level ), mParentItem( parentItem ) {}
 
     ~ScopeTraverser()
     {
-      if( mParentItem && !mParentItem->childCount() ) delete mParentItem;
+      if( mParentItem->type() == TemporaryItem && !mParentItem->childCount() ) delete mParentItem;
     }
 
     void process( DocEntry *entry )
     {
-      if ( mWidget->engine()->canSearch( entry ) &&
-           ( !mWidget->engine()->needsIndex( entry ) ||
-           entry->indexExists( Prefs::indexDirectory() ) ) ) {
-        ScopeItem *item = 0;
-        if ( mParentItem ) {
-          item = new ScopeItem( mParentItem, entry );
-        } else {
-          item = new ScopeItem( mWidget->listView(), entry );
-        }
+      if ( mEngine->canSearch( entry ) ) {
+        ScopeItem *item = new ScopeItem( mParentItem, entry );
         item->setOn( entry->searchEnabled() );
+        mItems.insert( entry, item );
       }
     }
 
     DocEntryTraverser *createChild( DocEntry *entry )
     {
-      if ( mLevel >= mNestingLevel ) {
+      if ( mLevel >= MaxNestingLevel ) {
         ++mLevel;
         return this;
       } else {
-        ScopeTraverser *t = new ScopeTraverser( mWidget, mLevel + 1 );
-        QTreeWidgetItem *item = 0;
-        if ( mParentItem ) {
-          item = new QTreeWidgetItem( mParentItem, QStringList() << entry->name() );
-        } else {
-          item = new QTreeWidgetItem( mWidget->listView(), QStringList() << entry->name() );
+        QTreeWidgetItem *item = mItems.value( entry );
+        if ( !item ) {
+          item = new QTreeWidgetItem( mParentItem, QStringList() << entry->name(), TemporaryItem );
         }
         item->setExpanded( true );
-        t->mParentItem = item;
+        ScopeTraverser *t = new ScopeTraverser( mEngine, item, mLevel + 1 );
         return t;
       }
     }
 
     DocEntryTraverser *parentTraverser()
     {
-      if ( mLevel > mNestingLevel ) return this;
+      if ( mLevel > MaxNestingLevel ) return this;
       else return mParent;
     }
 
     void deleteTraverser()
     {
-      if ( mLevel > mNestingLevel ) --mLevel;
+      if ( mLevel > MaxNestingLevel ) --mLevel;
       else delete this;
     }
 
   private:
-    SearchWidget *mWidget;
+    SearchEngine *mEngine;
     int mLevel;
     QTreeWidgetItem *mParentItem;
+    QHash<DocEntry *, QTreeWidgetItem *> mItems;
 
-    static int mNestingLevel;
+    enum { MaxNestingLevel = 2 };
+    enum { TemporaryItem = QTreeWidgetItem::UserType + 100 };
 };
 
-int ScopeTraverser::mNestingLevel = 2;
-
 void SearchWidget::searchIndexUpdated()
 {
   KSharedConfig::openConfig()->reparseConfiguration();
@@ -333,7 +309,7 @@ void SearchWidget::updateScopeList()
 {
   mScopeListView->clear();
 
-  ScopeTraverser t( this, 0 );
+  ScopeTraverser t( mEngine, mScopeListView->invisibleRootItem(), 0 );
   DocMetaInfo::self()->traverseEntries( &t );
 
   checkScope();
@@ -341,12 +317,12 @@ void SearchWidget::updateScopeList()
 
 void SearchWidget::scopeDoubleClicked( QTreeWidgetItem* item )
 {
-  if ( !item || item->type() != ScopeItem::rttiId() ) return;
+  if ( !item || item->type() != ScopeItem::ScopeItemType ) return;
   ScopeItem *scopeItem = static_cast<ScopeItem *>( item );
 
   QString searchUrl = scopeItem->entry()->search();
 
-  qDebug() << "DoubleClick: " << searchUrl;
+  khcDebug() << "DoubleClick: " << searchUrl;
 
   emit searchResult( searchUrl );
 }
@@ -358,7 +334,7 @@ void SearchWidget::scopeClicked( QTreeWidgetItem* )
   mScopeCombo->setCurrentIndex( ScopeCustom );
 }
 
-QString SearchWidget::scopeSelectionLabel( int id ) const
+QString SearchWidget::scopeSelectionLabel( int id )
 {
   switch( id ) {
     case ScopeCustom:
@@ -380,7 +356,7 @@ void SearchWidget::checkScope()
 
   QTreeWidgetItemIterator it( mScopeListView );
   while( (*it) ) {
-    if ( (*it)->type() == ScopeItem::rttiId() ) {
+    if ( (*it)->type() == ScopeItem::ScopeItemType ) {
       ScopeItem *item = static_cast<ScopeItem *>( (*it) );
       if ( item->isOn() ) {
         ++mScopeCount;
