@@ -21,20 +21,19 @@
 #include "toc.h"
 
 #include "docentry.h"
+#include "khc_debug.h"
 
-#include <KIconLoader>
 #include <KProcess>
 #include <KXmlGuiWindow>
 #include <QStatusBar>
 
-#include <QDir>
 #include <QFileInfo>
 #include <QTextStream>
-#include <QPixmap>
 
 #include <sys/stat.h>
 #include <QStandardPaths>
 #include <QApplication>
+#include <QIcon>
 
 using namespace KHC;
 
@@ -155,7 +154,7 @@ void TOC::buildCache()
     meinproc->setOutputChannelMode(KProcess::OnlyStderrChannel);
     meinproc->start();
     if (!meinproc->waitForStarted()) {
-        qWarning() << "could not start process" << meinproc->program();
+        khcWarning() << "could not start process" << meinproc->program();
         if (mainWindow && !m_alreadyWarned) {
             ; // add warning message box with don't display again option 
               // http://api.kde.org/4.0-api/kdelibs-apidocs/kdeui/html/classKDialog.html
@@ -171,8 +170,8 @@ void TOC::meinprocExited( int exitCode, QProcess::ExitStatus exitStatus)
     KXmlGuiWindow *mainWindow = dynamic_cast<KXmlGuiWindow *>( qobject_cast<QApplication*>(qApp)->activeWindow() );
 
     if ( exitStatus == QProcess::CrashExit || exitCode != 0 ) {
-        qWarning() << "running" << meinproc->program() << "failed with exitCode" << exitCode;
-        qWarning() << "stderr output:" << meinproc->readAllStandardError();
+        khcWarning() << "running" << meinproc->program() << "failed with exitCode" << exitCode;
+        khcWarning() << "stderr output:" << meinproc->readAllStandardError();
         if (mainWindow && !m_alreadyWarned) {
             ; // add warning message box with don't display again option 
               // http://api.kde.org/4.0-api/kdelibs-apidocs/kdeui/html/classKDialog.html
@@ -295,7 +294,7 @@ TOCSectionItem::TOCSectionItem( TOC *toc, TOCChapterItem *parent, QTreeWidgetIte
     : TOCItem( toc, parent, after, title ),
     m_name( name )
 {
-    setIcon( 0, SmallIcon( "text-plain" ) );
+    setIcon( 0, QIcon::fromTheme( "text-plain" ) );
     entry()->setUrl(url());
 }
 
